library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity lcd is
	 generic( N: natural := 40000);
    Port ( clk 		: in std_logic;
           reset 		: in std_logic;
			  DB			: in std_logic_vector(7 downto 0);
			  W			: in std_logic;
			  Ready	 	: inout std_logic;
           lcd_rs 	: out std_logic;
           lcd_rw 	: out std_logic;
           lcd_e 		: out std_logic;
           lcd_db 	: out std_logic_vector(7 downto 0));
end lcd;

architecture Behavioral of lcd is

component power_up is
	 generic( N: natural);
    Port ( clk 				: in std_logic;
	 		  reset				: in std_logic;
			  counter_enable	: in std_logic;
           data 				: out std_logic_vector(7 downto 0);
			  count	 			: out std_logic_vector(15 downto 0);
           done 				: out std_logic
			  );
end component;

signal data 			: std_logic_vector(7 downto 0);
signal counter 		: std_logic_vector(15 downto 0);
signal init_done		: std_logic;

signal WriteDataState	: std_logic;
signal WriteC0State		: std_logic;

signal bitcounter 		: std_logic_vector(3 downto 0);
signal counter_enable	: std_logic;
--**************************************************************************
-- STATE MACHINE SIGNAL DECLARATION:
type StateType is (
							Init, 
							Standby, 
							WriteData,
							WriteC0  --nineth character 
							);

signal CurrentState, NextState : StateType;
--**************************************************************************

begin

	power_up_module: power_up
	generic map (N)
	port map (  clk 				=> clk,
	 		  		reset				=> reset,
					counter_enable => Ready,
           		data 				=> data,
			  		count	 			=> counter,
           		done 				=> init_done);


COMB: process(CurrentState, counter, W, init_done)
begin
	case CurrentState is


		when Init =>
			if(init_done = '1') then		  --  if 16 mS have elapsed,
				NextState <= Standby;
			else NextState <= Init;
			end if;

		when Standby =>
			if(W = '1') then		  --  if 16 mS have elapsed,
				NextState <= WriteData;
			else NextState <= Standby;
			end if;

		when WriteData =>
			if(counter >= N) then		  --  if 16 mS have elapsed,
				if(bitcounter = 15) then
					NextState <= WriteC0;
				else
					NextState <= Standby;
				end if;
			else NextState <= WriteData;
			end if;

		when WriteC0 =>
			if(counter >= N) then		  --  if 16 mS have elapsed,
				NextState <= Standby;
			else NextState <= WriteC0;
			end if;


	end case;
end process COMB;

SEQ: process(reset, clk)
begin
	if(reset = '1') then
		CurrentState <= Init;
	elsif (clk'event and clk = '1') then
		CurrentState <= NextState;
	end if;
end process SEQ;


with CurrentState select
	Ready <= '1' when Standby,
			   '0' when Others; 

with CurrentState select
	WriteDataState <= '1' when WriteData,
			   			'0' when Others; 

with CurrentState select
	WriteC0State <= '1' when WriteC0,
			   		 '0' when Others; 


--**************************************************************************
--  4 bit counter
--**************************************************************************
counter_enable <= '1' when ((counter = 5) and (WriteDataState = '1')) else '0';
process(reset, clk, counter_enable)
begin
	if(reset = '1') then
		bitcounter <= (others=>'0');
	elsif (clk'event and clk = '1') then
		if(counter_enable = '1') then
			bitcounter <= bitcounter + 1;
		end if;
	end if;
end process;

				
--**************************************************************
LCD_DB <= x"C0" when WriteC0State = '1' else
			   DB when init_done = '1' else 
				data;
LCD_RS <= '1' when (WriteDataState = '1') else '0';
LCD_RW <= '0';	-- write only
LCD_E	 <= '1' when (counter = 5) else '0';
		 


end Behavioral;
